#!/bin/sh

usage()
{
	(
	echo "Usage: $0 [-4] [-5] [-d] [-f] [-h] [-I] [-i] [-j 16] [-K] [-k] [-n] [-r] [-s $SRC] [-t] [-V] [-w] [-x]"
	echo " -4	build all code for mlx4"
	echo " -5	build all code for mlx5"
	echo " -d	enable debugging"
	echo " -f	build innova fpga code for mlx5"
	echo " -h	show usage"
	echo " -I	build all infiniband related user-space code"
	echo " -i	build all infiniband related kernel code"
	echo " -j X	use X threads when building"
        echo " -K X	Kernel ident"
	echo " -k	continue building on error(s)"
	echo " -n	only build, do not install"
	echo " -r	enable rate limiting"
	echo " -s X	set the FreeBSD source tree directory"
	echo " -t	build mlx5tool"
        echo " -V 	show build process"
	echo " -w	disabling warnings"
	echo " -x	no clean"
	) 1>&2
	exit 2
}

set +e
ARGS=`getopt 45dfhIij:K:knrs:tVwx $*`
# Make sure errors cause an exit
set -e

# Make sure we don't conflict with the kernel
FLAGS="WITHOUT_OFED=YES"

TBUILD=""
UBUILD=""
SKIPINSTALL="NO"
SHOW_BUILD="NO"
X_NO_CLEAN="NO"

SRC="/usr/src"
ARCH=$(uname -m)
KERN_CONFIG=$(uname -i)

set -- $ARGS
for i
do
	case "$i"
	in
	-4)
		FLAGS="$FLAGS CONFIG_BUILD_MLX4=YES"
		shift
		;;
	-5)
		FLAGS="$FLAGS CONFIG_BUILD_MLX5=YES"
		shift
		;;
	-d)
		FLAGS="$FLAGS DEBUG_FLAGS=-g"
		FLAGS="$FLAGS CONFIG_RATELIMIT_DEBUG=YES"
		shift
		;;
	-f)
		FLAGS="$FLAGS CONFIG_BUILD_FPGA=YES"
		shift
		;;
	-h)
		usage
		shift
		;;
	-I)
		UBUILD="YES"
		shift
		;;
	-i)
		FLAGS="$FLAGS CONFIG_BUILD_INFINIBAND=YES"
		shift
		;;
	-j)
		FLAGS="$FLAGS -j$2"
		shift
		shift
		;;
        -K)
                KERN_CONFIG=$2
                shift
		shift
                ;;
	-k)
		FLAGS="$FLAGS -k"
		shift
		;;
	-n)
		SKIPINSTALL=YES
		shift
		;;
	-r)
		FLAGS="$FLAGS CONFIG_RATELIMIT=YES"
		shift
		;;
	-s)
		SRC="$2"
		shift
		shift
		;;
	-t)
		TBUILD="YES"
		shift
		;;
        -V)
                SHOW_BUILD=YES
                shift
                ;;
	-w)
		FLAGS="$FLAGS WARNS=0"
		shift
		;;
	-x)
		X_NO_CLEAN="YES"
		shift
		;;
	--)
		shift
		break
		;;
	esac
done


# Install modules in /boot/kernel
FLAGS="$FLAGS KMODDIR=/boot/kernel"

# Set SYSDIR variable
FLAGS="$FLAGS SYSDIR=$SRC/sys"

# Set -m flag for make
FLAGS="$FLAGS -m $SRC/share/mk"

# Generate option files
KERN_BUILD_DIR=$(mktemp -d);
PWD_OLD=$PWD

FLAGS="$FLAGS KERNBUILDDIR=$KERN_BUILD_DIR"
cd $SRC/sys/$ARCH/conf
config -d "$KERN_BUILD_DIR" $KERN_CONFIG
cd $PWD_OLD

# Dump flags for debugging purposes
echo "FLAGS=$FLAGS"

# Get the driver version
if [ -e driver_version.txt ]; then
    DRV_VER=$(cat driver_version.txt)
elif [ -d .git ]; then
    CUR_BRANCH=$(git rev-parse --abbrev-ref HEAD)
    LAST_HASH=$(git log -1 --format="%h")
    DRV_VER=$(echo $CUR_BRANCH-$LAST_HASH-devel)
else
    echo "Couldn't retrive driver version:"
    echo "Missing driver_version.txt file, and not git repo"
    exit 1
fi
FLAGS="$FLAGS EXT_DRV_VER=$DRV_VER"

if [ $SHOW_BUILD != "YES" ]; then
    FLAGS="$FLAGS -s "
fi

echo "The FreeBSD source tree is at $SRC ..."
if [ $X_NO_CLEAN != "YES" ]; then
make -C sys/modules $FLAGS clean cleandepend
make -C sys/modules $FLAGS depend
fi
make -C sys/modules $FLAGS all
if [ $SKIPINSTALL != "YES" ]; then
    make -C sys/modules $FLAGS install
fi

if [ "$UBUILD" = "YES" ]; then
    make -C contrib/ofed $FLAGS TESTBUILD=YES all-install
fi

if [ "$TBUILD" = "YES" ]; then
    make -C usr.sbin/mlx5tool $FLAGS CFLAGS=-I$(pwd) BINDIR=/usr/sbin clean cleandepend
    make -C usr.sbin/mlx5tool $FLAGS CFLAGS=-I$(pwd) BINDIR=/usr/sbin depend
    make -C usr.sbin/mlx5tool $FLAGS CFLAGS=-I$(pwd) BINDIR=/usr/sbin all
    if [ $SKIPINSTALL != "YES" ]; then
        make -C usr.sbin/mlx5tool $FLAGS CFLAGS=-I$(pwd) BINDIR=/usr/sbin install
    fi
fi

if [ $SKIPINSTALL != "YES" ]; then
    if [ -d scripts ]; then
        /bin/cp -v scripts/* $DESTDIR/usr/bin/
    fi
fi

rm -rd $KERN_BUILD_DIR
